package com.sf.service.gateway.mapper;

import com.sf.service.gateway.domain.GatewayServer;
import org.apache.ibatis.annotations.Param;

import java.util.Collection;
import java.util.List;

/**
 * 服务管理Mapper接口
 * 
 * @author zoukun
 * @date 2024-04-22
 */
public interface GatewayServerMapper 
{
    /**
     * 查询服务管理
     * 
     * @param id 服务管理主键
     * @return 服务管理
     */
    public GatewayServer selectGatewayServerById(Long id);

    /**
     * 查询服务管理列表
     * 
     * @param gatewayServer 服务管理
     * @return 服务管理集合
     */
    public List<GatewayServer> selectGatewayServerList(GatewayServer gatewayServer);

    /**
     * 新增服务管理
     * 
     * @param gatewayServer 服务管理
     * @return 结果
     */
    public int insertGatewayServer(GatewayServer gatewayServer);

    /**
     * 修改服务管理
     * 
     * @param gatewayServer 服务管理
     * @return 结果
     */
    public int updateGatewayServer(GatewayServer gatewayServer);

    /**
     * 删除服务管理
     * 
     * @param id 服务管理主键
     * @return 结果
     */
    public int deleteGatewayServerById(Long id);

    /**
     * 批量删除服务管理
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteGatewayServerByIds(Long[] ids);

    int updateGatewayServerStatusByIds(@Param("ids") Collection<Long> ids, @Param("status") String status);

    List<GatewayServer> selectGatewayServerByIds(@Param("ids") Collection<Long> ids);

    List<GatewayServer> selectGatewayServerByRouteIds(@Param("routeIds") Collection<Long> routeIds);
}
